# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Folder.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Folder' not in _M_omero.model.__dict__:
    _M_omero.model._t_Folder = IcePy.declareClass('::omero::model::Folder')
    _M_omero.model._t_FolderPrx = IcePy.declareProxy('::omero::model::Folder')

if 'FolderImageLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderImageLink = IcePy.declareClass('::omero::model::FolderImageLink')
    _M_omero.model._t_FolderImageLinkPrx = IcePy.declareProxy('::omero::model::FolderImageLink')

if 'Image' not in _M_omero.model.__dict__:
    _M_omero.model._t_Image = IcePy.declareClass('::omero::model::Image')
    _M_omero.model._t_ImagePrx = IcePy.declareProxy('::omero::model::Image')

if 'FolderRoiLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderRoiLink = IcePy.declareClass('::omero::model::FolderRoiLink')
    _M_omero.model._t_FolderRoiLinkPrx = IcePy.declareProxy('::omero::model::FolderRoiLink')

if 'Roi' not in _M_omero.model.__dict__:
    _M_omero.model._t_Roi = IcePy.declareClass('::omero::model::Roi')
    _M_omero.model._t_RoiPrx = IcePy.declareProxy('::omero::model::Roi')

if 'FolderAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderAnnotationLink = IcePy.declareClass('::omero::model::FolderAnnotationLink')
    _M_omero.model._t_FolderAnnotationLinkPrx = IcePy.declareProxy('::omero::model::FolderAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_FolderChildFoldersSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderChildFoldersSeq = IcePy.defineSequence('::omero::model::FolderChildFoldersSeq', (), _M_omero.model._t_Folder)

if '_t_FolderImageLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderImageLinksSeq = IcePy.defineSequence('::omero::model::FolderImageLinksSeq', (), _M_omero.model._t_FolderImageLink)

if '_t_FolderLinkedImageSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderLinkedImageSeq = IcePy.defineSequence('::omero::model::FolderLinkedImageSeq', (), _M_omero.model._t_Image)

if '_t_FolderRoiLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderRoiLinksSeq = IcePy.defineSequence('::omero::model::FolderRoiLinksSeq', (), _M_omero.model._t_FolderRoiLink)

if '_t_FolderLinkedRoiSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderLinkedRoiSeq = IcePy.defineSequence('::omero::model::FolderLinkedRoiSeq', (), _M_omero.model._t_Roi)

if '_t_FolderAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderAnnotationLinksSeq = IcePy.defineSequence('::omero::model::FolderAnnotationLinksSeq', (), _M_omero.model._t_FolderAnnotationLink)

if '_t_FolderLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FolderLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::FolderLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Folder' not in _M_omero.model.__dict__:
    _M_omero.model.Folder = Ice.createTempClass()
    class Folder(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _childFoldersSeq=None, _childFoldersLoaded=False, _parentFolder=None, _imageLinksSeq=None, _imageLinksLoaded=False, _imageLinksCountPerOwner=None, _roiLinksSeq=None, _roiLinksLoaded=False, _roiLinksCountPerOwner=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _name=None, _description=None):
            if Ice.getType(self) == _M_omero.model.Folder:
                raise RuntimeError('omero.model.Folder is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._childFoldersSeq = _childFoldersSeq
            self._childFoldersLoaded = _childFoldersLoaded
            self._parentFolder = _parentFolder
            self._imageLinksSeq = _imageLinksSeq
            self._imageLinksLoaded = _imageLinksLoaded
            self._imageLinksCountPerOwner = _imageLinksCountPerOwner
            self._roiLinksSeq = _roiLinksSeq
            self._roiLinksLoaded = _roiLinksLoaded
            self._roiLinksCountPerOwner = _roiLinksCountPerOwner
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner
            self._name = _name
            self._description = _description

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Folder', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::Folder'

        def ice_staticId():
            return '::omero::model::Folder'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def unloadChildFolders(self, current=None):
            pass

        def sizeOfChildFolders(self, current=None):
            pass

        def copyChildFolders(self, current=None):
            pass

        def addChildFolders(self, target, current=None):
            pass

        def addAllChildFoldersSet(self, targets, current=None):
            pass

        def removeChildFolders(self, theTarget, current=None):
            pass

        def removeAllChildFoldersSet(self, targets, current=None):
            pass

        def clearChildFolders(self, current=None):
            pass

        def reloadChildFolders(self, toCopy, current=None):
            pass

        def getParentFolder(self, current=None):
            pass

        def setParentFolder(self, theParentFolder, current=None):
            pass

        def unloadImageLinks(self, current=None):
            pass

        def sizeOfImageLinks(self, current=None):
            pass

        def copyImageLinks(self, current=None):
            pass

        def addFolderImageLink(self, target, current=None):
            pass

        def addAllFolderImageLinkSet(self, targets, current=None):
            pass

        def removeFolderImageLink(self, theTarget, current=None):
            pass

        def removeAllFolderImageLinkSet(self, targets, current=None):
            pass

        def clearImageLinks(self, current=None):
            pass

        def reloadImageLinks(self, toCopy, current=None):
            pass

        def getImageLinksCountPerOwner(self, current=None):
            pass

        def linkImage(self, addition, current=None):
            pass

        def addFolderImageLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFolderImageLink(self, removal, current=None):
            pass

        def unlinkImage(self, removal, current=None):
            pass

        def removeFolderImageLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedImageList(self, current=None):
            pass

        def unloadRoiLinks(self, current=None):
            pass

        def sizeOfRoiLinks(self, current=None):
            pass

        def copyRoiLinks(self, current=None):
            pass

        def addFolderRoiLink(self, target, current=None):
            pass

        def addAllFolderRoiLinkSet(self, targets, current=None):
            pass

        def removeFolderRoiLink(self, theTarget, current=None):
            pass

        def removeAllFolderRoiLinkSet(self, targets, current=None):
            pass

        def clearRoiLinks(self, current=None):
            pass

        def reloadRoiLinks(self, toCopy, current=None):
            pass

        def getRoiLinksCountPerOwner(self, current=None):
            pass

        def linkRoi(self, addition, current=None):
            pass

        def addFolderRoiLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFolderRoiLink(self, removal, current=None):
            pass

        def unlinkRoi(self, removal, current=None):
            pass

        def removeFolderRoiLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedRoiList(self, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addFolderAnnotationLink(self, target, current=None):
            pass

        def addAllFolderAnnotationLinkSet(self, targets, current=None):
            pass

        def removeFolderAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllFolderAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addFolderAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFolderAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeFolderAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def getName(self, current=None):
            pass

        def setName(self, theName, current=None):
            pass

        def getDescription(self, current=None):
            pass

        def setDescription(self, theDescription, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Folder)

        __repr__ = __str__

    _M_omero.model.FolderPrx = Ice.createTempClass()
    class FolderPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Folder._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Folder._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Folder._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Folder._op_setVersion.end(self, _r)

        def unloadChildFolders(self, _ctx=None):
            return _M_omero.model.Folder._op_unloadChildFolders.invoke(self, ((), _ctx))

        def begin_unloadChildFolders(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unloadChildFolders.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadChildFolders(self, _r):
            return _M_omero.model.Folder._op_unloadChildFolders.end(self, _r)

        def sizeOfChildFolders(self, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfChildFolders.invoke(self, ((), _ctx))

        def begin_sizeOfChildFolders(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfChildFolders.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfChildFolders(self, _r):
            return _M_omero.model.Folder._op_sizeOfChildFolders.end(self, _r)

        def copyChildFolders(self, _ctx=None):
            return _M_omero.model.Folder._op_copyChildFolders.invoke(self, ((), _ctx))

        def begin_copyChildFolders(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_copyChildFolders.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyChildFolders(self, _r):
            return _M_omero.model.Folder._op_copyChildFolders.end(self, _r)

        def addChildFolders(self, target, _ctx=None):
            return _M_omero.model.Folder._op_addChildFolders.invoke(self, ((target, ), _ctx))

        def begin_addChildFolders(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addChildFolders.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addChildFolders(self, _r):
            return _M_omero.model.Folder._op_addChildFolders.end(self, _r)

        def addAllChildFoldersSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_addAllChildFoldersSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllChildFoldersSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addAllChildFoldersSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllChildFoldersSet(self, _r):
            return _M_omero.model.Folder._op_addAllChildFoldersSet.end(self, _r)

        def removeChildFolders(self, theTarget, _ctx=None):
            return _M_omero.model.Folder._op_removeChildFolders.invoke(self, ((theTarget, ), _ctx))

        def begin_removeChildFolders(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeChildFolders.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeChildFolders(self, _r):
            return _M_omero.model.Folder._op_removeChildFolders.end(self, _r)

        def removeAllChildFoldersSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_removeAllChildFoldersSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllChildFoldersSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeAllChildFoldersSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllChildFoldersSet(self, _r):
            return _M_omero.model.Folder._op_removeAllChildFoldersSet.end(self, _r)

        def clearChildFolders(self, _ctx=None):
            return _M_omero.model.Folder._op_clearChildFolders.invoke(self, ((), _ctx))

        def begin_clearChildFolders(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_clearChildFolders.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearChildFolders(self, _r):
            return _M_omero.model.Folder._op_clearChildFolders.end(self, _r)

        def reloadChildFolders(self, toCopy, _ctx=None):
            return _M_omero.model.Folder._op_reloadChildFolders.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadChildFolders(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_reloadChildFolders.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadChildFolders(self, _r):
            return _M_omero.model.Folder._op_reloadChildFolders.end(self, _r)

        def getParentFolder(self, _ctx=None):
            return _M_omero.model.Folder._op_getParentFolder.invoke(self, ((), _ctx))

        def begin_getParentFolder(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getParentFolder.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getParentFolder(self, _r):
            return _M_omero.model.Folder._op_getParentFolder.end(self, _r)

        def setParentFolder(self, theParentFolder, _ctx=None):
            return _M_omero.model.Folder._op_setParentFolder.invoke(self, ((theParentFolder, ), _ctx))

        def begin_setParentFolder(self, theParentFolder, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_setParentFolder.begin(self, ((theParentFolder, ), _response, _ex, _sent, _ctx))

        def end_setParentFolder(self, _r):
            return _M_omero.model.Folder._op_setParentFolder.end(self, _r)

        def unloadImageLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_unloadImageLinks.invoke(self, ((), _ctx))

        def begin_unloadImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unloadImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadImageLinks(self, _r):
            return _M_omero.model.Folder._op_unloadImageLinks.end(self, _r)

        def sizeOfImageLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfImageLinks.invoke(self, ((), _ctx))

        def begin_sizeOfImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfImageLinks(self, _r):
            return _M_omero.model.Folder._op_sizeOfImageLinks.end(self, _r)

        def copyImageLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_copyImageLinks.invoke(self, ((), _ctx))

        def begin_copyImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_copyImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyImageLinks(self, _r):
            return _M_omero.model.Folder._op_copyImageLinks.end(self, _r)

        def addFolderImageLink(self, target, _ctx=None):
            return _M_omero.model.Folder._op_addFolderImageLink.invoke(self, ((target, ), _ctx))

        def begin_addFolderImageLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderImageLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFolderImageLink(self, _r):
            return _M_omero.model.Folder._op_addFolderImageLink.end(self, _r)

        def addAllFolderImageLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderImageLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFolderImageLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderImageLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFolderImageLinkSet(self, _r):
            return _M_omero.model.Folder._op_addAllFolderImageLinkSet.end(self, _r)

        def removeFolderImageLink(self, theTarget, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderImageLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFolderImageLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderImageLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFolderImageLink(self, _r):
            return _M_omero.model.Folder._op_removeFolderImageLink.end(self, _r)

        def removeAllFolderImageLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderImageLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFolderImageLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderImageLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFolderImageLinkSet(self, _r):
            return _M_omero.model.Folder._op_removeAllFolderImageLinkSet.end(self, _r)

        def clearImageLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_clearImageLinks.invoke(self, ((), _ctx))

        def begin_clearImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_clearImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearImageLinks(self, _r):
            return _M_omero.model.Folder._op_clearImageLinks.end(self, _r)

        def reloadImageLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Folder._op_reloadImageLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadImageLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_reloadImageLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadImageLinks(self, _r):
            return _M_omero.model.Folder._op_reloadImageLinks.end(self, _r)

        def getImageLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Folder._op_getImageLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getImageLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getImageLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getImageLinksCountPerOwner(self, _r):
            return _M_omero.model.Folder._op_getImageLinksCountPerOwner.end(self, _r)

        def linkImage(self, addition, _ctx=None):
            return _M_omero.model.Folder._op_linkImage.invoke(self, ((addition, ), _ctx))

        def begin_linkImage(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkImage.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkImage(self, _r):
            return _M_omero.model.Folder._op_linkImage.end(self, _r)

        def addFolderImageLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_addFolderImageLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFolderImageLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderImageLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFolderImageLinkToBoth(self, _r):
            return _M_omero.model.Folder._op_addFolderImageLinkToBoth.end(self, _r)

        def findFolderImageLink(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_findFolderImageLink.invoke(self, ((removal, ), _ctx))

        def begin_findFolderImageLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_findFolderImageLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFolderImageLink(self, _r):
            return _M_omero.model.Folder._op_findFolderImageLink.end(self, _r)

        def unlinkImage(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_unlinkImage.invoke(self, ((removal, ), _ctx))

        def begin_unlinkImage(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unlinkImage.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkImage(self, _r):
            return _M_omero.model.Folder._op_unlinkImage.end(self, _r)

        def removeFolderImageLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderImageLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFolderImageLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderImageLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFolderImageLinkFromBoth(self, _r):
            return _M_omero.model.Folder._op_removeFolderImageLinkFromBoth.end(self, _r)

        def linkedImageList(self, _ctx=None):
            return _M_omero.model.Folder._op_linkedImageList.invoke(self, ((), _ctx))

        def begin_linkedImageList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkedImageList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedImageList(self, _r):
            return _M_omero.model.Folder._op_linkedImageList.end(self, _r)

        def unloadRoiLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_unloadRoiLinks.invoke(self, ((), _ctx))

        def begin_unloadRoiLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unloadRoiLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadRoiLinks(self, _r):
            return _M_omero.model.Folder._op_unloadRoiLinks.end(self, _r)

        def sizeOfRoiLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfRoiLinks.invoke(self, ((), _ctx))

        def begin_sizeOfRoiLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfRoiLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfRoiLinks(self, _r):
            return _M_omero.model.Folder._op_sizeOfRoiLinks.end(self, _r)

        def copyRoiLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_copyRoiLinks.invoke(self, ((), _ctx))

        def begin_copyRoiLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_copyRoiLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyRoiLinks(self, _r):
            return _M_omero.model.Folder._op_copyRoiLinks.end(self, _r)

        def addFolderRoiLink(self, target, _ctx=None):
            return _M_omero.model.Folder._op_addFolderRoiLink.invoke(self, ((target, ), _ctx))

        def begin_addFolderRoiLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderRoiLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFolderRoiLink(self, _r):
            return _M_omero.model.Folder._op_addFolderRoiLink.end(self, _r)

        def addAllFolderRoiLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderRoiLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFolderRoiLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderRoiLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFolderRoiLinkSet(self, _r):
            return _M_omero.model.Folder._op_addAllFolderRoiLinkSet.end(self, _r)

        def removeFolderRoiLink(self, theTarget, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderRoiLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFolderRoiLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderRoiLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFolderRoiLink(self, _r):
            return _M_omero.model.Folder._op_removeFolderRoiLink.end(self, _r)

        def removeAllFolderRoiLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderRoiLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFolderRoiLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderRoiLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFolderRoiLinkSet(self, _r):
            return _M_omero.model.Folder._op_removeAllFolderRoiLinkSet.end(self, _r)

        def clearRoiLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_clearRoiLinks.invoke(self, ((), _ctx))

        def begin_clearRoiLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_clearRoiLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearRoiLinks(self, _r):
            return _M_omero.model.Folder._op_clearRoiLinks.end(self, _r)

        def reloadRoiLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Folder._op_reloadRoiLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadRoiLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_reloadRoiLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadRoiLinks(self, _r):
            return _M_omero.model.Folder._op_reloadRoiLinks.end(self, _r)

        def getRoiLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Folder._op_getRoiLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getRoiLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getRoiLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRoiLinksCountPerOwner(self, _r):
            return _M_omero.model.Folder._op_getRoiLinksCountPerOwner.end(self, _r)

        def linkRoi(self, addition, _ctx=None):
            return _M_omero.model.Folder._op_linkRoi.invoke(self, ((addition, ), _ctx))

        def begin_linkRoi(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkRoi.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkRoi(self, _r):
            return _M_omero.model.Folder._op_linkRoi.end(self, _r)

        def addFolderRoiLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_addFolderRoiLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFolderRoiLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderRoiLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFolderRoiLinkToBoth(self, _r):
            return _M_omero.model.Folder._op_addFolderRoiLinkToBoth.end(self, _r)

        def findFolderRoiLink(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_findFolderRoiLink.invoke(self, ((removal, ), _ctx))

        def begin_findFolderRoiLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_findFolderRoiLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFolderRoiLink(self, _r):
            return _M_omero.model.Folder._op_findFolderRoiLink.end(self, _r)

        def unlinkRoi(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_unlinkRoi.invoke(self, ((removal, ), _ctx))

        def begin_unlinkRoi(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unlinkRoi.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkRoi(self, _r):
            return _M_omero.model.Folder._op_unlinkRoi.end(self, _r)

        def removeFolderRoiLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderRoiLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFolderRoiLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderRoiLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFolderRoiLinkFromBoth(self, _r):
            return _M_omero.model.Folder._op_removeFolderRoiLinkFromBoth.end(self, _r)

        def linkedRoiList(self, _ctx=None):
            return _M_omero.model.Folder._op_linkedRoiList.invoke(self, ((), _ctx))

        def begin_linkedRoiList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkedRoiList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedRoiList(self, _r):
            return _M_omero.model.Folder._op_linkedRoiList.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Folder._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Folder._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Folder._op_copyAnnotationLinks.end(self, _r)

        def addFolderAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Folder._op_addFolderAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addFolderAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFolderAnnotationLink(self, _r):
            return _M_omero.model.Folder._op_addFolderAnnotationLink.end(self, _r)

        def addAllFolderAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFolderAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addAllFolderAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFolderAnnotationLinkSet(self, _r):
            return _M_omero.model.Folder._op_addAllFolderAnnotationLinkSet.end(self, _r)

        def removeFolderAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFolderAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFolderAnnotationLink(self, _r):
            return _M_omero.model.Folder._op_removeFolderAnnotationLink.end(self, _r)

        def removeAllFolderAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFolderAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeAllFolderAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFolderAnnotationLinkSet(self, _r):
            return _M_omero.model.Folder._op_removeAllFolderAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Folder._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Folder._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Folder._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Folder._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Folder._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Folder._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Folder._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Folder._op_linkAnnotation.end(self, _r)

        def addFolderAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_addFolderAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFolderAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_addFolderAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFolderAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Folder._op_addFolderAnnotationLinkToBoth.end(self, _r)

        def findFolderAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_findFolderAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findFolderAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_findFolderAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFolderAnnotationLink(self, _r):
            return _M_omero.model.Folder._op_findFolderAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Folder._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Folder._op_unlinkAnnotation.end(self, _r)

        def removeFolderAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFolderAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_removeFolderAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFolderAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Folder._op_removeFolderAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Folder._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Folder._op_linkedAnnotationList.end(self, _r)

        def getName(self, _ctx=None):
            return _M_omero.model.Folder._op_getName.invoke(self, ((), _ctx))

        def begin_getName(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getName.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getName(self, _r):
            return _M_omero.model.Folder._op_getName.end(self, _r)

        def setName(self, theName, _ctx=None):
            return _M_omero.model.Folder._op_setName.invoke(self, ((theName, ), _ctx))

        def begin_setName(self, theName, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_setName.begin(self, ((theName, ), _response, _ex, _sent, _ctx))

        def end_setName(self, _r):
            return _M_omero.model.Folder._op_setName.end(self, _r)

        def getDescription(self, _ctx=None):
            return _M_omero.model.Folder._op_getDescription.invoke(self, ((), _ctx))

        def begin_getDescription(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_getDescription.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDescription(self, _r):
            return _M_omero.model.Folder._op_getDescription.end(self, _r)

        def setDescription(self, theDescription, _ctx=None):
            return _M_omero.model.Folder._op_setDescription.invoke(self, ((theDescription, ), _ctx))

        def begin_setDescription(self, theDescription, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Folder._op_setDescription.begin(self, ((theDescription, ), _response, _ex, _sent, _ctx))

        def end_setDescription(self, _r):
            return _M_omero.model.Folder._op_setDescription.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.FolderPrx.ice_checkedCast(proxy, '::omero::model::Folder', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.FolderPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Folder'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_FolderPrx = IcePy.defineProxy('::omero::model::Folder', FolderPrx)

    _M_omero.model._t_Folder = IcePy.defineClass('::omero::model::Folder', Folder, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_childFoldersSeq', (), _M_omero.model._t_FolderChildFoldersSeq, False, 0),
        ('_childFoldersLoaded', (), IcePy._t_bool, False, 0),
        ('_parentFolder', (), _M_omero.model._t_Folder, False, 0),
        ('_imageLinksSeq', (), _M_omero.model._t_FolderImageLinksSeq, False, 0),
        ('_imageLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_imageLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_roiLinksSeq', (), _M_omero.model._t_FolderRoiLinksSeq, False, 0),
        ('_roiLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_roiLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_FolderAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_name', (), _M_omero._t_RString, False, 0),
        ('_description', (), _M_omero._t_RString, False, 0)
    ))
    Folder._ice_type = _M_omero.model._t_Folder

    Folder._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Folder._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Folder._op_unloadChildFolders = IcePy.Operation('unloadChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_sizeOfChildFolders = IcePy.Operation('sizeOfChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Folder._op_copyChildFolders = IcePy.Operation('copyChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderChildFoldersSeq, False, 0), ())
    Folder._op_addChildFolders = IcePy.Operation('addChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_addAllChildFoldersSet = IcePy.Operation('addAllChildFoldersSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderChildFoldersSeq, False, 0),), (), None, ())
    Folder._op_removeChildFolders = IcePy.Operation('removeChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_removeAllChildFoldersSet = IcePy.Operation('removeAllChildFoldersSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderChildFoldersSeq, False, 0),), (), None, ())
    Folder._op_clearChildFolders = IcePy.Operation('clearChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_reloadChildFolders = IcePy.Operation('reloadChildFolders', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_getParentFolder = IcePy.Operation('getParentFolder', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Folder, False, 0), ())
    Folder._op_setParentFolder = IcePy.Operation('setParentFolder', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_unloadImageLinks = IcePy.Operation('unloadImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_sizeOfImageLinks = IcePy.Operation('sizeOfImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Folder._op_copyImageLinks = IcePy.Operation('copyImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderImageLinksSeq, False, 0), ())
    Folder._op_addFolderImageLink = IcePy.Operation('addFolderImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLink, False, 0),), (), None, ())
    Folder._op_addAllFolderImageLinkSet = IcePy.Operation('addAllFolderImageLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLinksSeq, False, 0),), (), None, ())
    Folder._op_removeFolderImageLink = IcePy.Operation('removeFolderImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLink, False, 0),), (), None, ())
    Folder._op_removeAllFolderImageLinkSet = IcePy.Operation('removeAllFolderImageLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLinksSeq, False, 0),), (), None, ())
    Folder._op_clearImageLinks = IcePy.Operation('clearImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_reloadImageLinks = IcePy.Operation('reloadImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_getImageLinksCountPerOwner = IcePy.Operation('getImageLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Folder._op_linkImage = IcePy.Operation('linkImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), ((), _M_omero.model._t_FolderImageLink, False, 0), ())
    Folder._op_addFolderImageLinkToBoth = IcePy.Operation('addFolderImageLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_findFolderImageLink = IcePy.Operation('findFolderImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), ((), _M_omero.model._t_FolderImageLinksSeq, False, 0), ())
    Folder._op_unlinkImage = IcePy.Operation('unlinkImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), None, ())
    Folder._op_removeFolderImageLinkFromBoth = IcePy.Operation('removeFolderImageLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderImageLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_linkedImageList = IcePy.Operation('linkedImageList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderLinkedImageSeq, False, 0), ())
    Folder._op_unloadRoiLinks = IcePy.Operation('unloadRoiLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_sizeOfRoiLinks = IcePy.Operation('sizeOfRoiLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Folder._op_copyRoiLinks = IcePy.Operation('copyRoiLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderRoiLinksSeq, False, 0), ())
    Folder._op_addFolderRoiLink = IcePy.Operation('addFolderRoiLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLink, False, 0),), (), None, ())
    Folder._op_addAllFolderRoiLinkSet = IcePy.Operation('addAllFolderRoiLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLinksSeq, False, 0),), (), None, ())
    Folder._op_removeFolderRoiLink = IcePy.Operation('removeFolderRoiLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLink, False, 0),), (), None, ())
    Folder._op_removeAllFolderRoiLinkSet = IcePy.Operation('removeAllFolderRoiLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLinksSeq, False, 0),), (), None, ())
    Folder._op_clearRoiLinks = IcePy.Operation('clearRoiLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_reloadRoiLinks = IcePy.Operation('reloadRoiLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_getRoiLinksCountPerOwner = IcePy.Operation('getRoiLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Folder._op_linkRoi = IcePy.Operation('linkRoi', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Roi, False, 0),), (), ((), _M_omero.model._t_FolderRoiLink, False, 0), ())
    Folder._op_addFolderRoiLinkToBoth = IcePy.Operation('addFolderRoiLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_findFolderRoiLink = IcePy.Operation('findFolderRoiLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Roi, False, 0),), (), ((), _M_omero.model._t_FolderRoiLinksSeq, False, 0), ())
    Folder._op_unlinkRoi = IcePy.Operation('unlinkRoi', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Roi, False, 0),), (), None, ())
    Folder._op_removeFolderRoiLinkFromBoth = IcePy.Operation('removeFolderRoiLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderRoiLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_linkedRoiList = IcePy.Operation('linkedRoiList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderLinkedRoiSeq, False, 0), ())
    Folder._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Folder._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderAnnotationLinksSeq, False, 0), ())
    Folder._op_addFolderAnnotationLink = IcePy.Operation('addFolderAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLink, False, 0),), (), None, ())
    Folder._op_addAllFolderAnnotationLinkSet = IcePy.Operation('addAllFolderAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLinksSeq, False, 0),), (), None, ())
    Folder._op_removeFolderAnnotationLink = IcePy.Operation('removeFolderAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLink, False, 0),), (), None, ())
    Folder._op_removeAllFolderAnnotationLinkSet = IcePy.Operation('removeAllFolderAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLinksSeq, False, 0),), (), None, ())
    Folder._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Folder._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Folder, False, 0),), (), None, ())
    Folder._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Folder._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_FolderAnnotationLink, False, 0), ())
    Folder._op_addFolderAnnotationLinkToBoth = IcePy.Operation('addFolderAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_findFolderAnnotationLink = IcePy.Operation('findFolderAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_FolderAnnotationLinksSeq, False, 0), ())
    Folder._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Folder._op_removeFolderAnnotationLinkFromBoth = IcePy.Operation('removeFolderAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FolderAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Folder._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FolderLinkedAnnotationSeq, False, 0), ())
    Folder._op_getName = IcePy.Operation('getName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Folder._op_setName = IcePy.Operation('setName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Folder._op_getDescription = IcePy.Operation('getDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Folder._op_setDescription = IcePy.Operation('setDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.Folder = Folder
    del Folder

    _M_omero.model.FolderPrx = FolderPrx
    del FolderPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# webclient_gateway
#
# Copyright (c) 2008-2011 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2012
#
# Version: 1.0
#

from builtins import str
from future.utils import native_str
from past.builtins import basestring
from builtins import object
import logging
import json

try:
    long
except:
    # Python 3
    long = int

logger = logging.getLogger(__name__)


class GatewayConfig(object):

    """
    Global Gateway configuration:

    - :attr:`IMG_RDEFNS`:  a namespace for annotations linked on images holding
                           the default rendering settings object id.
    - :attr:`IMG_ROPTSNS`: a namespace for annotations linked on images holding
                           default rendering options that don't get saved in
                           the rendering settings.
    """

    def __init__(self):
        self.IMG_RDEFNS = None
        self.IMG_ROPTSNS = None


class ServiceOptsDict(dict):

    def __new__(cls, *args, **kwargs):
        return super(ServiceOptsDict, cls).__new__(cls, *args, **kwargs)

    def __init__(self, data=None, *args, **kwargs):
        if data is None:
            data = dict()
        if len(kwargs) > 0:
            for key, val in dict(*args, **kwargs).items():
                self[key] = val
        if isinstance(data, dict):
            for key in data:
                item = data[key]
                if self._testItem(item):
                    self[key] = str(item)
                else:
                    logger.debug(
                        "None or non- string, unicode or numeric type"
                        "values are ignored, (%r, %r)" % (key, item))
        else:
            raise AttributeError(
                "%s argument (%r:%s) must be a dictionary"
                % (self.__class__.__name__, data, type(data)))

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__,
                             super(ServiceOptsDict, self).__repr__())

    def __setitem__(self, key, item):
        """Set key to value as string."""
        if self._testItem(item):
            super(ServiceOptsDict, self).__setitem__(key, native_str(item))
            logger.debug("Setting %r to %r" % (key, item))
        else:
            raise AttributeError(
                "%s argument (%r:%s) must be a string, unicode or numeric type"
                % (self.__class__.__name__, item, type(item)))

    def __getitem__(self, key):
        """
        Return the value for key if key is in the dictionary.
        Raises a KeyError if key is not in the map.
        """
        try:
            return super(ServiceOptsDict, self).__getitem__(key)
        except KeyError:
            raise KeyError("Key %r not found in %r" % (key, self))

    def __delitem__(self, key):
        """
        Remove dict[key] from dict.
        Raises a KeyError if key is not in the map.
        """
        super(ServiceOptsDict, self).__delitem__(key)
        logger.debug("Deleting %r" % (key))

    def copy(self):
        """Returns a copy of this object."""
        return self.__class__(self)

    def clear(self):
        """Remove all items from the dictionary."""
        super(ServiceOptsDict, self).clear()

    def get(self, key, default=None):
        """
        Return the value for key if key is in the dictionary, else default.
        If default is not given, it defaults to None, so that this method
        never raises a KeyError.
        """
        try:
            return self.__getitem__(key)
        except KeyError:
            return default

    def set(self, key, value):
        """Set key to value as string."""
        return self.__setitem__(key, value)

    def getOmeroGroup(self):
        return self.get('omero.group')

    def setOmeroGroup(self, value=None):
        if value is not None:
            self.set('omero.group', value)
        else:
            try:
                del self['omero.group']
            except KeyError:
                logger.debug("Key 'omero.group' not found in %r" % self)

    def getOmeroUser(self):
        return self.get('omero.user')

    def setOmeroUser(self, value=None):
        if value is not None:
            self.set('omero.user', value)
        else:
            try:
                del self['omero.user']
            except KeyError:
                logger.debug("Key 'omero.user' not found in %r" % self)

    def getOmeroShare(self):
        return self.get('omero.share')

    def setOmeroShare(self, value=None):
        if value is not None:
            self.set('omero.share', value)
        else:
            try:
                del self['omero.share']
            except KeyError:  # pragma: no cover
                logger.debug("Key 'omero.share' not found in %r" % self)

    def _testItem(self, item):
        if item is not None and not isinstance(item, bool) and \
            (isinstance(item, basestring) or
             isinstance(item, int) or
             isinstance(item, long) or
             isinstance(item, float)):
            return True
        return False


def toBoolean(val):
    """
    Get the boolean value of the provided input.

        If the value is a boolean return the value.
        Otherwise check to see if the value is in
        ["true", "yes", "y", "t", "1"]
        and returns True if value is in the list
    """

    if val is True or val is False:
        return val

    trueItems = ["true", "yes", "y", "t", "1", "on"]

    return str(val).strip().lower() in trueItems


def propertiesToDict(m, prefix=None):
    """
    Convert omero properties to nested dictionary, skipping common prefix
    """

    nested_dict = {}
    for item, value in m.items():
        d = nested_dict
        if prefix is not None:
            item = item.replace(prefix, "")
        items = item.split('.')
        for key in items[:-1]:
            d = d.setdefault(key, {})
        try:
            if value.strip().lower() in ('true', 'false'):
                d[items[-1]] = toBoolean(value)
            else:
                d[items[-1]] = json.loads(value)
        except:
            d[items[-1]] = value
    return nested_dict



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_FilterType_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class FilterTypeI(_omero_model.FilterType):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "value",
          "details",
      ])
      _field_info = _field_info_type(
          value=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      VALUE =  "ome.model.enums.FilterType_value"
      DETAILS =  "ome.model.enums.FilterType_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          pass

      def __init__(self, id=None, loaded=None):
          super(FilterTypeI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadValue( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  True ;
      def isMutable(self, current = None):
          return  False ;
      def isAnnotated(self, current = None):
          return  False ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = FilterTypeI()
            copy._id = self._id;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return FilterTypeI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def unloadValue(self, ):
          self._valueLoaded = False
          self._value = None;

      def getValue(self, current = None):
          self.errorIfUnloaded()
          return self._value

      def setValue(self, _value, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.value.wrapper is not None:
              if _value is not None:
                  _value = self._field_info.value.wrapper(_value)
          self._value = _value
          pass


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.FilterTypeI = FilterTypeI



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Channel.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'StatsInfo' not in _M_omero.model.__dict__:
    _M_omero.model._t_StatsInfo = IcePy.declareClass('::omero::model::StatsInfo')
    _M_omero.model._t_StatsInfoPrx = IcePy.declareProxy('::omero::model::StatsInfo')

if 'LogicalChannel' not in _M_omero.model.__dict__:
    _M_omero.model._t_LogicalChannel = IcePy.declareClass('::omero::model::LogicalChannel')
    _M_omero.model._t_LogicalChannelPrx = IcePy.declareProxy('::omero::model::LogicalChannel')

if 'Pixels' not in _M_omero.model.__dict__:
    _M_omero.model._t_Pixels = IcePy.declareClass('::omero::model::Pixels')
    _M_omero.model._t_PixelsPrx = IcePy.declareProxy('::omero::model::Pixels')

if 'ChannelAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ChannelAnnotationLink = IcePy.declareClass('::omero::model::ChannelAnnotationLink')
    _M_omero.model._t_ChannelAnnotationLinkPrx = IcePy.declareProxy('::omero::model::ChannelAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_ChannelAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ChannelAnnotationLinksSeq = IcePy.defineSequence('::omero::model::ChannelAnnotationLinksSeq', (), _M_omero.model._t_ChannelAnnotationLink)

if '_t_ChannelLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ChannelLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::ChannelLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Channel' not in _M_omero.model.__dict__:
    _M_omero.model.Channel = Ice.createTempClass()
    class Channel(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _statsInfo=None, _red=None, _green=None, _blue=None, _alpha=None, _lookupTable=None, _logicalChannel=None, _pixels=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Channel:
                raise RuntimeError('omero.model.Channel is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._statsInfo = _statsInfo
            self._red = _red
            self._green = _green
            self._blue = _blue
            self._alpha = _alpha
            self._lookupTable = _lookupTable
            self._logicalChannel = _logicalChannel
            self._pixels = _pixels
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Channel', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::Channel'

        def ice_staticId():
            return '::omero::model::Channel'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getStatsInfo(self, current=None):
            pass

        def setStatsInfo(self, theStatsInfo, current=None):
            pass

        def getRed(self, current=None):
            pass

        def setRed(self, theRed, current=None):
            pass

        def getGreen(self, current=None):
            pass

        def setGreen(self, theGreen, current=None):
            pass

        def getBlue(self, current=None):
            pass

        def setBlue(self, theBlue, current=None):
            pass

        def getAlpha(self, current=None):
            pass

        def setAlpha(self, theAlpha, current=None):
            pass

        def getLookupTable(self, current=None):
            pass

        def setLookupTable(self, theLookupTable, current=None):
            pass

        def getLogicalChannel(self, current=None):
            pass

        def setLogicalChannel(self, theLogicalChannel, current=None):
            pass

        def getPixels(self, current=None):
            pass

        def setPixels(self, thePixels, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addChannelAnnotationLink(self, target, current=None):
            pass

        def addAllChannelAnnotationLinkSet(self, targets, current=None):
            pass

        def removeChannelAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllChannelAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addChannelAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findChannelAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeChannelAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Channel)

        __repr__ = __str__

    _M_omero.model.ChannelPrx = Ice.createTempClass()
    class ChannelPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Channel._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Channel._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Channel._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Channel._op_setVersion.end(self, _r)

        def getStatsInfo(self, _ctx=None):
            return _M_omero.model.Channel._op_getStatsInfo.invoke(self, ((), _ctx))

        def begin_getStatsInfo(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getStatsInfo.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getStatsInfo(self, _r):
            return _M_omero.model.Channel._op_getStatsInfo.end(self, _r)

        def setStatsInfo(self, theStatsInfo, _ctx=None):
            return _M_omero.model.Channel._op_setStatsInfo.invoke(self, ((theStatsInfo, ), _ctx))

        def begin_setStatsInfo(self, theStatsInfo, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setStatsInfo.begin(self, ((theStatsInfo, ), _response, _ex, _sent, _ctx))

        def end_setStatsInfo(self, _r):
            return _M_omero.model.Channel._op_setStatsInfo.end(self, _r)

        def getRed(self, _ctx=None):
            return _M_omero.model.Channel._op_getRed.invoke(self, ((), _ctx))

        def begin_getRed(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getRed.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRed(self, _r):
            return _M_omero.model.Channel._op_getRed.end(self, _r)

        def setRed(self, theRed, _ctx=None):
            return _M_omero.model.Channel._op_setRed.invoke(self, ((theRed, ), _ctx))

        def begin_setRed(self, theRed, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setRed.begin(self, ((theRed, ), _response, _ex, _sent, _ctx))

        def end_setRed(self, _r):
            return _M_omero.model.Channel._op_setRed.end(self, _r)

        def getGreen(self, _ctx=None):
            return _M_omero.model.Channel._op_getGreen.invoke(self, ((), _ctx))

        def begin_getGreen(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getGreen.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getGreen(self, _r):
            return _M_omero.model.Channel._op_getGreen.end(self, _r)

        def setGreen(self, theGreen, _ctx=None):
            return _M_omero.model.Channel._op_setGreen.invoke(self, ((theGreen, ), _ctx))

        def begin_setGreen(self, theGreen, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setGreen.begin(self, ((theGreen, ), _response, _ex, _sent, _ctx))

        def end_setGreen(self, _r):
            return _M_omero.model.Channel._op_setGreen.end(self, _r)

        def getBlue(self, _ctx=None):
            return _M_omero.model.Channel._op_getBlue.invoke(self, ((), _ctx))

        def begin_getBlue(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getBlue.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getBlue(self, _r):
            return _M_omero.model.Channel._op_getBlue.end(self, _r)

        def setBlue(self, theBlue, _ctx=None):
            return _M_omero.model.Channel._op_setBlue.invoke(self, ((theBlue, ), _ctx))

        def begin_setBlue(self, theBlue, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setBlue.begin(self, ((theBlue, ), _response, _ex, _sent, _ctx))

        def end_setBlue(self, _r):
            return _M_omero.model.Channel._op_setBlue.end(self, _r)

        def getAlpha(self, _ctx=None):
            return _M_omero.model.Channel._op_getAlpha.invoke(self, ((), _ctx))

        def begin_getAlpha(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getAlpha.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAlpha(self, _r):
            return _M_omero.model.Channel._op_getAlpha.end(self, _r)

        def setAlpha(self, theAlpha, _ctx=None):
            return _M_omero.model.Channel._op_setAlpha.invoke(self, ((theAlpha, ), _ctx))

        def begin_setAlpha(self, theAlpha, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setAlpha.begin(self, ((theAlpha, ), _response, _ex, _sent, _ctx))

        def end_setAlpha(self, _r):
            return _M_omero.model.Channel._op_setAlpha.end(self, _r)

        def getLookupTable(self, _ctx=None):
            return _M_omero.model.Channel._op_getLookupTable.invoke(self, ((), _ctx))

        def begin_getLookupTable(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getLookupTable.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLookupTable(self, _r):
            return _M_omero.model.Channel._op_getLookupTable.end(self, _r)

        def setLookupTable(self, theLookupTable, _ctx=None):
            return _M_omero.model.Channel._op_setLookupTable.invoke(self, ((theLookupTable, ), _ctx))

        def begin_setLookupTable(self, theLookupTable, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setLookupTable.begin(self, ((theLookupTable, ), _response, _ex, _sent, _ctx))

        def end_setLookupTable(self, _r):
            return _M_omero.model.Channel._op_setLookupTable.end(self, _r)

        def getLogicalChannel(self, _ctx=None):
            return _M_omero.model.Channel._op_getLogicalChannel.invoke(self, ((), _ctx))

        def begin_getLogicalChannel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getLogicalChannel.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLogicalChannel(self, _r):
            return _M_omero.model.Channel._op_getLogicalChannel.end(self, _r)

        def setLogicalChannel(self, theLogicalChannel, _ctx=None):
            return _M_omero.model.Channel._op_setLogicalChannel.invoke(self, ((theLogicalChannel, ), _ctx))

        def begin_setLogicalChannel(self, theLogicalChannel, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setLogicalChannel.begin(self, ((theLogicalChannel, ), _response, _ex, _sent, _ctx))

        def end_setLogicalChannel(self, _r):
            return _M_omero.model.Channel._op_setLogicalChannel.end(self, _r)

        def getPixels(self, _ctx=None):
            return _M_omero.model.Channel._op_getPixels.invoke(self, ((), _ctx))

        def begin_getPixels(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getPixels.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPixels(self, _r):
            return _M_omero.model.Channel._op_getPixels.end(self, _r)

        def setPixels(self, thePixels, _ctx=None):
            return _M_omero.model.Channel._op_setPixels.invoke(self, ((thePixels, ), _ctx))

        def begin_setPixels(self, thePixels, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_setPixels.begin(self, ((thePixels, ), _response, _ex, _sent, _ctx))

        def end_setPixels(self, _r):
            return _M_omero.model.Channel._op_setPixels.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Channel._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Channel._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Channel._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Channel._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Channel._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Channel._op_copyAnnotationLinks.end(self, _r)

        def addChannelAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Channel._op_addChannelAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addChannelAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_addChannelAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addChannelAnnotationLink(self, _r):
            return _M_omero.model.Channel._op_addChannelAnnotationLink.end(self, _r)

        def addAllChannelAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Channel._op_addAllChannelAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllChannelAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_addAllChannelAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllChannelAnnotationLinkSet(self, _r):
            return _M_omero.model.Channel._op_addAllChannelAnnotationLinkSet.end(self, _r)

        def removeChannelAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Channel._op_removeChannelAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeChannelAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_removeChannelAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeChannelAnnotationLink(self, _r):
            return _M_omero.model.Channel._op_removeChannelAnnotationLink.end(self, _r)

        def removeAllChannelAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Channel._op_removeAllChannelAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllChannelAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_removeAllChannelAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllChannelAnnotationLinkSet(self, _r):
            return _M_omero.model.Channel._op_removeAllChannelAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Channel._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Channel._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Channel._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Channel._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Channel._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Channel._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Channel._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Channel._op_linkAnnotation.end(self, _r)

        def addChannelAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Channel._op_addChannelAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addChannelAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_addChannelAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addChannelAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Channel._op_addChannelAnnotationLinkToBoth.end(self, _r)

        def findChannelAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Channel._op_findChannelAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findChannelAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_findChannelAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findChannelAnnotationLink(self, _r):
            return _M_omero.model.Channel._op_findChannelAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Channel._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Channel._op_unlinkAnnotation.end(self, _r)

        def removeChannelAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Channel._op_removeChannelAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeChannelAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_removeChannelAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeChannelAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Channel._op_removeChannelAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Channel._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Channel._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Channel._op_linkedAnnotationList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.ChannelPrx.ice_checkedCast(proxy, '::omero::model::Channel', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.ChannelPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Channel'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_ChannelPrx = IcePy.defineProxy('::omero::model::Channel', ChannelPrx)

    _M_omero.model._t_Channel = IcePy.declareClass('::omero::model::Channel')

    _M_omero.model._t_Channel = IcePy.defineClass('::omero::model::Channel', Channel, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_statsInfo', (), _M_omero.model._t_StatsInfo, False, 0),
        ('_red', (), _M_omero._t_RInt, False, 0),
        ('_green', (), _M_omero._t_RInt, False, 0),
        ('_blue', (), _M_omero._t_RInt, False, 0),
        ('_alpha', (), _M_omero._t_RInt, False, 0),
        ('_lookupTable', (), _M_omero._t_RString, False, 0),
        ('_logicalChannel', (), _M_omero.model._t_LogicalChannel, False, 0),
        ('_pixels', (), _M_omero.model._t_Pixels, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_ChannelAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Channel._ice_type = _M_omero.model._t_Channel

    Channel._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Channel._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Channel._op_getStatsInfo = IcePy.Operation('getStatsInfo', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_StatsInfo, False, 0), ())
    Channel._op_setStatsInfo = IcePy.Operation('setStatsInfo', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_StatsInfo, False, 0),), (), None, ())
    Channel._op_getRed = IcePy.Operation('getRed', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Channel._op_setRed = IcePy.Operation('setRed', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Channel._op_getGreen = IcePy.Operation('getGreen', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Channel._op_setGreen = IcePy.Operation('setGreen', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Channel._op_getBlue = IcePy.Operation('getBlue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Channel._op_setBlue = IcePy.Operation('setBlue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Channel._op_getAlpha = IcePy.Operation('getAlpha', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Channel._op_setAlpha = IcePy.Operation('setAlpha', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Channel._op_getLookupTable = IcePy.Operation('getLookupTable', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Channel._op_setLookupTable = IcePy.Operation('setLookupTable', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Channel._op_getLogicalChannel = IcePy.Operation('getLogicalChannel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_LogicalChannel, False, 0), ())
    Channel._op_setLogicalChannel = IcePy.Operation('setLogicalChannel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_LogicalChannel, False, 0),), (), None, ())
    Channel._op_getPixels = IcePy.Operation('getPixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Pixels, False, 0), ())
    Channel._op_setPixels = IcePy.Operation('setPixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Pixels, False, 0),), (), None, ())
    Channel._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Channel._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Channel._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ChannelAnnotationLinksSeq, False, 0), ())
    Channel._op_addChannelAnnotationLink = IcePy.Operation('addChannelAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLink, False, 0),), (), None, ())
    Channel._op_addAllChannelAnnotationLinkSet = IcePy.Operation('addAllChannelAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLinksSeq, False, 0),), (), None, ())
    Channel._op_removeChannelAnnotationLink = IcePy.Operation('removeChannelAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLink, False, 0),), (), None, ())
    Channel._op_removeAllChannelAnnotationLinkSet = IcePy.Operation('removeAllChannelAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLinksSeq, False, 0),), (), None, ())
    Channel._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Channel._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Channel, False, 0),), (), None, ())
    Channel._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Channel._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ChannelAnnotationLink, False, 0), ())
    Channel._op_addChannelAnnotationLinkToBoth = IcePy.Operation('addChannelAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Channel._op_findChannelAnnotationLink = IcePy.Operation('findChannelAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ChannelAnnotationLinksSeq, False, 0), ())
    Channel._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Channel._op_removeChannelAnnotationLinkFromBoth = IcePy.Operation('removeChannelAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChannelAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Channel._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ChannelLinkedAnnotationSeq, False, 0), ())

    _M_omero.model.Channel = Channel
    del Channel

    _M_omero.model.ChannelPrx = ChannelPrx
    del ChannelPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2014 Glencoe Software, Inc. All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Automatic configuration of memory settings for Java servers.
"""
from __future__ import division

from builtins import str
from builtins import range
from future.utils import bytes_to_native_str
from past.utils import old_div
from builtins import object
from past.builtins import basestring
from shlex import split

import logging

LOGGER = logging.getLogger("omero.install.jvmcfg")


def strip_dict(map, prefix=("omero", "jvmcfg"), suffix=(), limit=1):
    """
    For the given dictionary, return a copy of the
    dictionary where all entries not matching the
    prefix, suffix, and limit have been removed and
    where all remaining keys have had the prefix and
    suffix stripped. The limit describes the number
    of elements that are allowed in the new key after
    stripping prefix and suffix.
    """
    if isinstance(prefix, basestring):
        prefix = tuple(prefix.split("."))
    if isinstance(suffix, basestring):
        suffix = tuple(suffix.split("."))
    rv = dict()
    if not map:
        return dict()

    def __strip_dict(k, v, prefix, suffix, rv):
        key = tuple(k.split("."))
        ksz = len(key)
        psz = len(prefix)
        ssz = len(suffix)
        if ksz <= (psz + ssz):
            return  # No way to strip if smaller
        if key[0:psz] == prefix and key[ksz-ssz:] == suffix:
            newkey = key[psz:ksz-ssz]
            if len(newkey) == limit:
                newkey = ".".join(newkey)
                rv[newkey] = v

    for k, v in list(map.items()):
        __strip_dict(k, v, prefix, suffix, rv)
    return rv


class StrategyRegistry(dict):

    def __init__(self, *args, **kwargs):
        super(dict, self).__init__(*args, **kwargs)


STRATEGY_REGISTRY = StrategyRegistry()


class Settings(object):
    """
    Container for the config options found in etc/grid/config.xml
    """

    def __init__(self, server_values=None, global_values=None):
        if server_values is None:
            self.__server = dict()
        else:
            self.__server = server_values
        if global_values is None:
            self.__global = dict()
        else:
            self.__global = global_values
        self.__static = {
            "strategy": PercentStrategy,
            "append": "",
            "perm_gen": "128m",
            "heap_dump": "off",
            "heap_size": "512m",
            "system_memory": None,
            "max_system_memory": "48000",
            "min_system_memory": "3414",
        }
        self.__manual = dict()

    def __getattr__(self, key):
        return self.lookup(key)

    def lookup(self, key, default=None):
        if key in self.__manual:
            return self.__manual[key]
        elif key in self.__server:
            return self.__server[key]
        elif key in self.__global:
            return self.__global[key]
        elif key in self.__static:
            return self.__static[key]
        else:
            return default

    def overwrite(self, key, value, always=False):
        if self.was_set(key) and not always:
            # Then we leave it as the user requested
            return
        else:
            self.__manual[key] = value

    def was_set(self, key):
        return key in self.__server or key in self.__global

    def get_strategy(self):
        return STRATEGY_REGISTRY.get(self.strategy, self.strategy)

    def __str__(self):
        rv = dict()
        rv.update(self.__server)
        rv.update(self.__global)
        if not rv:
            rv = ""
        return 'Settings(%s)' % rv


class Strategy(object):
    """
    Strategy for calculating memory settings. Primary
    class of the memory module.
    """

    def __init__(self, name, settings=None):
        """
        'name' argument should likely be one of:
        ('blitz', 'indexer', 'pixeldata', 'repository')
        """
        if settings is None:
            settings = Settings()
        self.name = name
        self.settings = settings
        if type(self) == Strategy:
            raise Exception("Must subclass!")

    # Memory helpers

    def system_memory_mb(self):
        """
        Returns a tuple, in MB, of available, active, and total memory.

        "total" memory is found by calling to first a Python library
        (if installed) and otherwise a Java class. If
        "system_memory" is set, it will short-circuit both methods.

        "active" memory is set to "total" but limited by "min_system_memory"
        and "max_system_memory".

        "available" may not be accurate, and in some cases will be
        set to total.
        """

        available, total = None, None
        if self.settings.system_memory is not None:
            total = int(self.settings.system_memory)
            available = total
        else:
            pymem = self._system_memory_mb_psutil()
            if pymem is not None:
                available, total = pymem
            else:
                available, total = self._system_memory_mb_java()

        max_system_memory = int(self.settings.max_system_memory)
        min_system_memory = int(self.settings.min_system_memory)
        active = max(min(total, max_system_memory), min_system_memory)
        return available, active, total

    def _system_memory_mb_psutil(self):
        try:
            import psutil
            pymem = psutil.virtual_memory()
            return (old_div(pymem.free,1000000), old_div(pymem.total,1000000))
        except ImportError:
            LOGGER.debug("No psutil installed")
            return None

    def _system_memory_mb_java(self):
        import omero.cli
        import omero.java

        # Copied from db.py. Needs better dir detection
        cwd = omero.cli.CLI().dir
        jars = str(cwd / "lib" / "server") + "/*"
        cmd = ["ome.services.util.JvmSettingsCheck", "--psutil"]
        p = omero.java.popen(["-cp", str(jars)] + cmd)
        o, e = list(map(bytes_to_native_str, p.communicate()))

        if p.poll() != 0:
            LOGGER.warn("Failed to invoke java:\nout:%s\nerr:%s",
                        o, e)

        rv = dict()
        for line in o.split("\n"):
            line = line.strip()
            if not line:
                continue
            parts = line.split(":")
            if len(parts) == 1:
                parts.append("")
            rv[parts[0]] = parts[1]

        try:
            free = old_div(int(rv["Free"]), 1000000)
        except:
            LOGGER.warn("Failed to parse Free from %s", rv)
            free = 2000

        try:
            total = old_div(int(rv["Total"]), 1000000)
        except:
            LOGGER.warn("Failed to parse Total from %s", rv)
            total = 4000

        return (free, total)

    # API Getters

    def get_heap_size(self, sz=None):
        if sz is None or self.settings.was_set("heap_size"):
            sz = self.settings.heap_size
        if str(sz).startswith("-X"):
            return sz
        else:
            try:
                sz = int(float(sz))
            except ValueError:
                pass
            rv = "-Xmx%s" % sz
            if rv[-1].lower() not in ("b", "k", "m", "g"):
                rv = "%sm" % rv
            return rv

    def get_heap_dump(self):
        hd = self.settings.heap_dump
        if hd == "off":
            return ""
        elif hd in ("on", "cwd", "tmp"):
            return "-XX:+HeapDumpOnOutOfMemoryError"

    def get_perm_gen(self):
        pg = self.settings.perm_gen
        if str(pg).startswith("-XX"):
            return pg
        else:
            return "-XX:MaxPermSize=%s" % pg

    def get_append(self):
        values = []
        if self.settings.heap_dump == "tmp":
            import tempfile
            tmp = tempfile.gettempdir()
            values.append("-XX:HeapDumpPath=%s" % tmp)
        return values + split(self.settings.append)

    def get_memory_settings(self):
        values = [
            self.get_heap_size(),
            self.get_heap_dump(),
            self.get_perm_gen(),
        ]
        if any([x.startswith("-XX:MaxPermSize") for x in values]):
            values.append("-XX:+IgnoreUnrecognizedVMOptions")
        values += self.get_append()
        return [x for x in values if x]


class ManualStrategy(Strategy):
    """
    Simplest strategy which assumes all values have
    been set and simply uses them or their defaults.
    """


class PercentStrategy(Strategy):
    """
    Strategy based on a percent of available memory.
    """

    PERCENT_DEFAULTS = (
        ("blitz", 15),
        ("pixeldata", 15),
        ("indexer", 10),
        ("repository", 10),
        ("other", 1),
    )

    def __init__(self, name, settings=None):
        super(PercentStrategy, self).__init__(name, settings)
        self.defaults = dict(self.PERCENT_DEFAULTS)
        self.use_active = True

    def get_heap_size(self):
        """
        Uses the results of the default settings of
        calculate_heap_size() as an argument to
        get_heap_size(), in other words some percent
        of the active memory.
        """
        sz = self.calculate_heap_size()
        return super(PercentStrategy, self).get_heap_size(sz)

    def get_percent(self):
        other = self.defaults.get("other", "1")
        default = self.defaults.get(self.name, other)
        percent = int(self.settings.lookup("percent", default))
        return percent

    def get_perm_gen(self):
        available, active, total = self.system_memory_mb()
        choice = self.use_active and active or total

        if choice <= 4000:
            if choice >= 2000:
                self.settings.overwrite("perm_gen", "256m")
        elif choice <= 8000:
            self.settings.overwrite("perm_gen", "512m")
        else:
            self.settings.overwrite("perm_gen", "1g")

        return super(PercentStrategy, self).get_perm_gen()

    def calculate_heap_size(self, method=None):
        """
        Re-calculates the appropriate heap size based on the
        value of get_percent(). The "active" memory returned
        by method() will be used by default, but can be modified
        to use "total" via the "use_active" flag.
        """
        if method is None:
            method = self.system_memory_mb

        available, active, total = method()
        choice = self.use_active and active or total

        percent = self.get_percent()
        calculated = int(choice * int(percent) / 100)
        return calculated

    def usage_table(self, min=10, max=20):
        total_mb = [2**x for x in range(min, max)]
        for total in total_mb:
            method = lambda: (total, total, total)
            yield total, self.calculate_heap_size(method)


STRATEGY_REGISTRY["manual"] = ManualStrategy
STRATEGY_REGISTRY["percent"] = PercentStrategy


def read_settings(template_xml):
    """
    Read the memory settings from the template file
    """

    rv = dict()
    for template in template_xml.findall("server-template"):
        for server in template.findall("server"):
            for option in server.findall("option"):
                o = option.text
                if o.startswith("-Xmx") | o.startswith("-XX"):
                    rv.setdefault(server.get('id'), []).append(o)
    return rv


def adjust_settings(config, template_xml,
                    blitz=None, indexer=None,
                    pixeldata=None, repository=None):
    """
    Takes an omero.config.ConfigXml object and adjusts
    the memory settings. Primary entry point to the
    memory module.
    """

    from xml.etree.ElementTree import Element
    from collections import defaultdict

    replacements = dict()
    options = dict()
    for template in template_xml.findall("server-template"):
        for server in template.findall("server"):
            for option in server.findall("option"):
                o = option.text
                if o.startswith("MEMORY:"):
                    options[o[7:]] = (server, option)
            for props in server.findall("properties"):
                for prop in props.findall("property"):
                    name = prop.attrib.get("name", "")
                    if name.startswith("REPLACEMENT:"):
                        replacements[name[12:]] = (server, prop)

    rv = defaultdict(list)
    m = config.as_map()
    loop = (("blitz", blitz), ("indexer", indexer),
            ("pixeldata", pixeldata), ("repository", repository))

    for name, StrategyType in loop:
        if name not in options:
            raise Exception(
                "Cannot find %s option. Make sure templates.xml was "
                "not copied from an older server" % name)

    for name, StrategyType in loop:
        specific = strip_dict(m, suffix=name)
        defaults = strip_dict(m)
        settings = Settings(specific, defaults)
        rv[name].append(settings)
        if StrategyType is None:
            StrategyType = settings.get_strategy()

        if not callable(StrategyType):
            raise Exception("Bad strategy: %s" % StrategyType)

        strategy = StrategyType(name, settings)
        settings = strategy.get_memory_settings()
        server, option = options[name]
        idx = 0
        for v in settings:
            rv[name].append(v)
            if idx == 0:
                option.text = v
            else:
                elem = Element("option")
                elem.text = v
                server.insert(idx, elem)
            idx += 1

        # Now we check for any other properties and
        # put them where the replacement should go.
        for k, v in list(m.items()):
            r = []
            suffix = ".%s" % name
            size = len(suffix)
            if k.endswith(suffix):
                k = k[:-size]
                r.append((k, v))

        server, replacement = replacements[name]
        idx = 0
        for k, v in r:
            if idx == 0:
                replacement.attrib["name"] = k
                replacement.attrib["value"] = v
            else:
                elem = Element("property", name=k, value=v)
                server.append(elem)

    return rv


def usage_charts(path,
                 min=0, max=20,
                 Strategy=PercentStrategy, name="blitz"):
    # See http://matplotlib.org/examples/pylab_examples/anscombe.html

    from pylab import array
    from pylab import axis
    from pylab import gca
    from pylab import subplot
    from pylab import plot
    from pylab import setp
    from pylab import savefig
    from pylab import text

    points = 200
    x = array([old_div(2 ** (old_div(x, points)), 1000)
               for x in range(min*points, max*points)])
    y_configs = (
        (Settings({}), 'A'),
        (Settings({"percent": "20"}), 'B'),
        (Settings({}), 'C'),
        (Settings({"max_system_memory": "10000"}), 'D'),
    )

    def f(cfg):
        s = Strategy(name, settings=cfg[0])
        y = []
        for total in x:
            method = lambda: (total, total, total)
            y.append(s.calculate_heap_size(method))
        return y

    y1 = f(y_configs[0])
    y2 = f(y_configs[1])
    y3 = f(y_configs[2])
    y4 = f(y_configs[3])

    axis_values = [0, 20, 0, 6]

    def ticks_f():
        setp(gca(), xticks=(8, 16), yticks=(2, 4))

    def text_f(which):
        cfg = y_configs[which]
        # s = cfg[0]
        txt = "%s" % (cfg[1],)
        text(2, 2, txt, fontsize=20)

    subplot(221)
    plot(x, y1)
    axis(axis_values)
    text_f(0)
    ticks_f()

    subplot(222)
    plot(x, y2)
    axis(axis_values)
    text_f(1)
    ticks_f()

    subplot(223)
    plot(x, y3)
    axis(axis_values)
    text_f(2)
    ticks_f()

    subplot(224)
    plot(x, y4)
    axis(axis_values)
    text_f(3)
    ticks_f()

    savefig(path)
